/* Tera Term
 Copyright(C) 1994-1996 T. Teranishi
 All rights reserved. */

/* TERATERM.EXE, TEK window */
#include "teraterm.h"
#include "tttypes.h"
#include "tektypes.h"
#include "ttwinman.h"
#include "ttcommon.h"
#include "keyboard.h"
#include "clipboar.h"
#include "ttdialog.h"
#include "teraprn.h"

#include "tek.h"

  static HMODULE HTTTEK = NULL;

typedef void (FAR PASCAL *PTEKInit)
  (PTEKVar tk, PTTSet ts);
typedef void (FAR PASCAL *PTEKResizeWindow)
  (PTEKVar tk, PTTSet ts, int W, int H);
typedef void (FAR PASCAL *PTEKChangeCaret)
  (PTEKVar tk, PTTSet ts);
typedef int (FAR PASCAL *PTEKParse)
  (PTEKVar tk, PTTSet ts, PComVar cv);
typedef void (FAR PASCAL *PTEKReportGIN)
  (PTEKVar tk, PTTSet ts, PComVar cv, BYTE KeyCode);
typedef void (FAR PASCAL *PTEKPaint)
  (PTEKVar tk, PTTSet ts, HDC PaintDC, PAINTSTRUCT *PaintInfo);
typedef void (FAR PASCAL *PTEKWMLButtonDown)
  (PTEKVar tk, PTTSet ts, PComVar cv, BOOL Popup, POINT pos);
typedef void (FAR PASCAL *PTEKWMLButtonUp)
  (PTEKVar tk, PTTSet ts);
typedef void (FAR PASCAL *PTEKWMMouseMove)
  (PTEKVar tk, PTTSet ts, POINT p);
typedef void (FAR PASCAL *PTEKWMSize)
  (PTEKVar tk, PTTSet ts, int W, int H, int cx, int cy);
typedef void (FAR PASCAL *PTEKCMCopy)
  (PTEKVar tk, PTTSet ts);
typedef void (FAR PASCAL *PTEKCMCopyScreen)
  (PTEKVar tk, PTTSet ts);
typedef void (FAR PASCAL *PTEKInitMenu)
  (PTEKVar tk, PComVar cv, HMENU SubMenu);
typedef void (FAR PASCAL *PTEKPrint)
  (PTEKVar tk, PTTSet ts, HDC PrintDC, BOOL SelFlag);
typedef void (FAR PASCAL *PTEKClearScreen)
  (PTEKVar tk);
typedef void (FAR PASCAL *PTEKSetupFont)
  (PTEKVar tk, PTTSet ts);
typedef void (FAR PASCAL *PTEKSetupWinDlg)
  (PTEKVar tk, PTTSet ts);
typedef void (FAR PASCAL *PTEKRestoreSetup)
  (PTEKVar tk, PTTSet ts);
typedef void (FAR PASCAL *PTEKEnd)
  (PTEKVar tk);
typedef void (FAR PASCAL *PTEKChangeMenu)
  (PTEKVar tk, PTTSet ts);

static PTEKInit TEKInit;
static PTEKResizeWindow TEKResizeWindow;
static PTEKChangeCaret TEKChangeCaret;
static PTEKParse TEKParse;
static PTEKReportGIN TEKReportGIN;
static PTEKPaint TEKPaint;
static PTEKWMLButtonDown TEKWMLButtonDown;
static PTEKWMLButtonUp TEKWMLButtonUp;
static PTEKWMMouseMove TEKWMMouseMove;
static PTEKWMSize TEKWMSize;
static PTEKCMCopy TEKCMCopy;
static PTEKCMCopyScreen TEKCMCopyScreen;
static PTEKInitMenu TEKInitMenu;
static PTEKPrint TEKPrint;
static PTEKClearScreen TEKClearScreen;
static PTEKSetupFont TEKSetupFont;
static PTEKSetupWinDlg TEKSetupWinDlg;
static PTEKRestoreSetup TEKRestoreSetup;
static PTEKEnd TEKEnd;
static PTEKChangeMenu TEKChangeMenu;

  BOOL LoadTTTEK();
  void FreeTTTEK();

#define IdTEKInit          1
#define IdTEKResizeWindow  2
#define IdTEKChangeCaret   3
#define IdTEKParse         4
#define IdTEKReportGIN     5
#define IdTEKPaint         6
#define IdTEKWMLButtonDown 7
#define IdTEKWMLButtonUp   8
#define IdTEKWMMouseMove   9
#define IdTEKWMSize        10
#define IdTEKCMCopy        11
#define IdTEKCMCopyScreen  12
#define IdTEKInitMenu      13
#define IdTEKPrint         14
#define IdTEKClearScreen   15
#define IdTEKSetupFont     16
#define IdTEKSetupWinDlg   17
#define IdTEKRestoreSetup  18
#define IdTEKEnd           19
#define IdTEKChangeMenu    20

BOOL LoadTTTEK()
{
  BOOL Err;

  if (HTTTEK != NULL) return TRUE;

  HTTTEK = LoadLibrary("TTPTEK.DLL");
  if (HTTTEK == NULL) return FALSE;

  Err = FALSE;
  TEKInit = (PTEKInit)GetProcAddress(HTTTEK,
    MAKEINTRESOURCE(IdTEKInit));
  if (TEKInit==NULL) Err = TRUE;
  
  TEKResizeWindow = (PTEKResizeWindow)GetProcAddress(HTTTEK,
    MAKEINTRESOURCE(IdTEKResizeWindow));
  if (TEKResizeWindow==NULL) Err = TRUE;

  TEKChangeCaret = (PTEKChangeCaret)GetProcAddress(HTTTEK,
    MAKEINTRESOURCE(IdTEKChangeCaret));
  if (TEKChangeCaret==NULL) Err = TRUE;

  TEKParse = (PTEKParse)GetProcAddress(HTTTEK,
    MAKEINTRESOURCE(IdTEKParse));
  if (TEKParse==NULL) Err = TRUE;

  TEKReportGIN = (PTEKReportGIN)GetProcAddress(HTTTEK,
    MAKEINTRESOURCE(IdTEKReportGIN));
  if (TEKReportGIN==NULL) Err = TRUE;

  TEKPaint = (PTEKPaint)GetProcAddress(HTTTEK,
    MAKEINTRESOURCE(IdTEKPaint));
  if (TEKPaint==NULL) Err = TRUE;

  TEKWMLButtonDown = (PTEKWMLButtonDown)GetProcAddress(HTTTEK,
    MAKEINTRESOURCE(IdTEKWMLButtonDown));
  if (TEKWMLButtonDown==NULL) Err = TRUE;

  TEKWMLButtonUp = (PTEKWMLButtonUp)GetProcAddress(HTTTEK,
    MAKEINTRESOURCE(IdTEKWMLButtonUp));
  if (TEKWMLButtonUp==NULL) Err = TRUE;

  TEKWMMouseMove = (PTEKWMMouseMove)GetProcAddress(HTTTEK,
    MAKEINTRESOURCE(IdTEKWMMouseMove));
  if (TEKWMMouseMove==NULL) Err = TRUE;

  TEKWMSize = (PTEKWMSize)GetProcAddress(HTTTEK,
    MAKEINTRESOURCE(IdTEKWMSize));
  if (TEKWMSize==NULL) Err = TRUE;

  TEKCMCopy = (PTEKCMCopy)GetProcAddress(HTTTEK,
    MAKEINTRESOURCE(IdTEKCMCopy));
  if (TEKCMCopy==NULL) Err = TRUE;

  TEKCMCopyScreen = (PTEKCMCopyScreen)GetProcAddress(HTTTEK,
    MAKEINTRESOURCE(IdTEKCMCopyScreen));
  if (TEKCMCopyScreen==NULL) Err = TRUE;

  TEKInitMenu = (PTEKInitMenu)GetProcAddress(HTTTEK,
    MAKEINTRESOURCE(IdTEKInitMenu));
  if (TEKInitMenu==NULL) Err = TRUE;

  TEKPrint = (PTEKPrint)GetProcAddress(HTTTEK,
    MAKEINTRESOURCE(IdTEKPrint));
  if (TEKPrint==NULL) Err = TRUE;

  TEKClearScreen = (PTEKClearScreen)GetProcAddress(HTTTEK,
    MAKEINTRESOURCE(IdTEKClearScreen));
  if (TEKClearScreen==NULL) Err = TRUE;

  TEKSetupFont = (PTEKSetupFont)GetProcAddress(HTTTEK,
    MAKEINTRESOURCE(IdTEKSetupFont));
  if (TEKSetupFont==NULL) Err = TRUE;

  TEKSetupWinDlg = (PTEKSetupWinDlg)GetProcAddress(HTTTEK,
    MAKEINTRESOURCE(IdTEKSetupWinDlg));
  if (TEKSetupWinDlg==NULL) Err = TRUE;

  TEKRestoreSetup = (PTEKRestoreSetup)GetProcAddress(HTTTEK,
    MAKEINTRESOURCE(IdTEKRestoreSetup));
  if (TEKRestoreSetup==NULL) Err = TRUE;

  TEKEnd = (PTEKEnd)GetProcAddress(HTTTEK,
    MAKEINTRESOURCE(IdTEKEnd));
  if (TEKEnd==NULL) Err = TRUE;

  TEKChangeMenu = (PTEKChangeMenu)GetProcAddress(HTTTEK,
    MAKEINTRESOURCE(IdTEKChangeMenu));
  if (TEKChangeMenu==NULL) Err = TRUE;

  if (Err)
  {
    FreeLibrary(HTTTEK);
    HTTTEK = NULL;
  }
  return (! Err);
}

void FreeTTTEK()
{
  if (HTTTEK!=NULL)
  {
    FreeLibrary(HTTTEK);
    HTTTEK = NULL;
  }
}

void TEKWinActivate()
{
  ActiveWin = IdTEK;
  if (HTEKWin==NULL)
  {
    pTEKWin = new CTEKWindow();
  }
  else {
    ShowWindow(HTEKWin,SW_SHOWNORMAL);
    SetFocus(HTEKWin);
  }
}

int TEKWinParse()
{
  if (pTEKWin != NULL)
    return ( ((CTEKWindow *)pTEKWin)->Parse() );
  else
    return (IdVT);
}

void TEKWinRestoreSetup()
{
  if (pTEKWin == NULL) return;
  ((CTEKWindow *)pTEKWin)->RestoreSetup();
}

/* ---------- TEK Window ------------ */

BEGIN_MESSAGE_MAP( CTEKWindow, CFrameWnd )
  //{{AFX_MSG_MAP( CTEKWindow )

  ON_WM_ACTIVATE()
  ON_WM_CHAR()
  ON_WM_DESTROY()
  ON_WM_GETMINMAXINFO()
  ON_WM_INITMENUPOPUP()
  ON_WM_KEYDOWN()
  ON_WM_KEYUP()
  ON_WM_LBUTTONDOWN()
  ON_WM_LBUTTONUP()
  ON_WM_MBUTTONUP()
  ON_WM_MOUSEMOVE()
  ON_WM_MOVE()
  ON_WM_PAINT()
  ON_WM_RBUTTONUP()
  ON_WM_SIZE()
  ON_WM_SYSCOMMAND()
  ON_WM_SYSKEYDOWN()
  ON_WM_SYSKEYUP()

  ON_MESSAGE(WM_USER+MsgChangeMenu,OnChangeMenu)
  ON_MESSAGE(WM_USER+MsgDlgHelp2,OnDlgHelp)

  ON_COMMAND(CM_TEKFILEPRINT,CMFilePrint)
  ON_COMMAND(CM_TEKFILEEXIT,CMFileExit)
  ON_COMMAND(CM_TEKEDITCOPY,CMEditCopy)
  ON_COMMAND(CM_TEKEDITCOPYSCREEN,CMEditCopyScreen)
  ON_COMMAND(CM_TEKEDITPASTE,CMEditPaste)
  ON_COMMAND(CM_TEKEDITPASTECR,CMEditPasteCR)
  ON_COMMAND(CM_TEKEDITCLEARSCREEN,CMEditClearScreen)
  ON_COMMAND(CM_TEKSETUPWINDOW,CMSetupWindow)
  ON_COMMAND(CM_TEKSETUPFONT,CMSetupFont)
  ON_COMMAND(CM_TEKVTWIN,CMVTWin)
  ON_COMMAND(CM_TEKHELPINDEX,CMHelpIndex)
  ON_COMMAND(CM_TEKHELPUSING,CMHelpUsing)
  ON_COMMAND(CM_TEKABOUT,CMAbout)

  //}}AFX_MSG_MAP
END_MESSAGE_MAP()

CTEKWindow::CTEKWindow()
{
  WNDCLASS wc;
  RECT rect;
  DWORD Style;

  if (! LoadTTTEK()) return;
  (*TEKInit)(&tk, &ts);

  if (ts.HideTitle>0)
    Style = WS_POPUP | WS_THICKFRAME | WS_BORDER;
  else
    Style = WS_CAPTION | WS_SYSMENU |
	    WS_MINIMIZEBOX | WS_BORDER | WS_THICKFRAME;

  wc.style = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc = AfxWndProc;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = 0;
  wc.hInstance = AfxGetInstanceHandle();
  wc.hIcon = LoadIcon(wc.hInstance, MAKEINTRESOURCE(IdTEKIcon));
  wc.hCursor = LoadCursor(NULL,IDC_ARROW);
  wc.hbrBackground = NULL;
  wc.lpszMenuName = NULL;
  wc.lpszClassName = "TEKWin32";

  RegisterClass(&wc);

  if (ts.TEKPos.x==CW_USEDEFAULT)
    rect = rectDefault;
  else {
    rect.left = ts.TEKPos.x;
    rect.top = ts.TEKPos.y;
    rect.right = rect.left + 600;
    rect.bottom = rect.top + 450;
  }
  Create("TEKWin32", "Tera Term", Style, rect, NULL, NULL);
//--------------------------------------------------------
  HTEKWin = GetSafeHwnd();
  if (HTEKWin == NULL) return;
  tk.HWin = HTEKWin;
  if (tk.MainMenu != NULL)
    ::SetMenu(tk.HWin,tk.MainMenu);

  ChangeTitle();

  ::GetWindowRect(tk.HWin,&tk.R);
  (*TEKResizeWindow)(&tk,&ts,
     tk.R.right-tk.R.left, tk.R.bottom-tk.R.top);

  if (ts.PopupMenu>0)
    ::PostMessage(HTEKWin,WM_USER+MsgChangeMenu,0,0);

  ::ShowWindow(tk.HWin, SW_SHOWNORMAL);
}

LRESULT CTEKWindow::DefWindowProc(UINT message, WPARAM wParam, LPARAM lParam)
{
  LRESULT Result;

  if (message == MsgDlgHelp)
  {
    OnDlgHelp(wParam,lParam);
    return 0;
  }
  else if ((ts.HideTitle>0) &&
           (message == WM_NCHITTEST))
  {
    Result = CWnd::DefWindowProc(message,wParam,lParam);
    if ((Result==HTCLIENT) && AltKey())
      Result = HTCAPTION;
    return Result;
  }

  return (CWnd::DefWindowProc(message,wParam,lParam));
}

int CTEKWindow::Parse()
{
  return ((*TEKParse)(&tk,&ts,&cv));
}

void CTEKWindow::RestoreSetup()
{
  (*TEKRestoreSetup)(&tk,&ts);
  ChangeTitle();
}

void CTEKWindow::OnActivate
  (UINT nState, CWnd* pWndOther, BOOL bMinimized)
{
  if (nState!=WA_INACTIVE)
  {
    tk.Active = TRUE;
    (*TEKChangeCaret)(&tk,&ts);
    ActiveWin = IdTEK;
  }
  else {
    DestroyCaret();
    tk.Active = FALSE;
  }
}

void CTEKWindow::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags)
{
  int i;
  char Code;

  if (TalkStatus!=IdTalkKeyb) return;
  Code = nChar;

  if (tk.GIN) (*TEKReportGIN)(&tk,&ts,&cv,Code);
  else
    for (i=1 ; i<=nRepCnt ; i++)
    {
      CommTextOut(&cv,&Code,1);
      if (ts.LocalEcho>0)
        CommTextEcho(&cv,&Code,1);
    }
}

void CTEKWindow::OnDestroy()
{
  CFrameWnd::OnDestroy();

  (*TEKEnd)(&tk);
  FreeTTTEK();
  HTEKWin = NULL;
  pTEKWin = NULL;
  ActiveWin = IdVT;
}

void CTEKWindow::OnGetMinMaxInfo(MINMAXINFO FAR* lpMMI)
{
  lpMMI->ptMaxSize.x = 10000;
  lpMMI->ptMaxSize.y = 10000;
  lpMMI->ptMaxTrackSize.x = 10000;
  lpMMI->ptMaxTrackSize.y = 10000;
}

void CTEKWindow::OnInitMenuPopup
  (CMenu* pPopupMenu,UINT nIndex, BOOL bSysmenu)
{
  (*TEKInitMenu)(&tk,&cv,pPopupMenu->m_hMenu);
}

void CTEKWindow::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
  KeyDown(HTEKWin,nChar,nRepCnt,nFlags & 0x1ff);
}

void CTEKWindow::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags)
{
  KeyUp(nChar);
}

void CTEKWindow::OnLButtonDown(UINT nFlags, CPoint point)
{
  BOOL Popup;
  POINT p;

  p.x = point.x;
  p.y = point.y;

  Popup = ControlKey() && (tk.MainMenu==NULL);
  (*TEKWMLButtonDown)(&tk,&ts,&cv,Popup,p);
  if (Popup)
  {
    ::ClientToScreen(tk.HWin, &p);
    TrackPopupMenu(tk.PopupBase,TPM_LEFTALIGN | TPM_LEFTBUTTON,
                   p.x,p.y,0,tk.HWin,NULL);
    DestroyMenu(tk.PopupBase);
    DestroyMenu(tk.PopupMenu);
  }
}

void CTEKWindow::OnLButtonUp(UINT nFlags, CPoint point)
{
  (*TEKWMLButtonUp)(&tk,&ts);
}

void CTEKWindow::OnMButtonUp(UINT nFlags, CPoint point)
{
  OnRButtonUp(nFlags,point);
}

void CTEKWindow::OnMouseMove(UINT nFlags, CPoint point)
{
  POINT p;

  p.x = point.x;
  p.y = point.y;
  (*TEKWMMouseMove)(&tk,&ts,p);
}

void CTEKWindow::OnMove(int x, int y)
{
  RECT R;

  ::GetWindowRect(HTEKWin,&R);
  ts.TEKPos.x = R.left;
  ts.TEKPos.y = R.top;
}

void CTEKWindow::OnPaint()
{
  PAINTSTRUCT ps;
  CDC *cdc;
  HDC PaintDC;

  cdc = BeginPaint(&ps);
  PaintDC = cdc->GetSafeHdc();

  (*TEKPaint)(&tk,&ts,PaintDC,&ps);

  EndPaint(&ps);
}

void CTEKWindow::OnRButtonUp(UINT nFlags, CPoint point)
{
  CBStartPaste(tk.HWin,FALSE);
}

void CTEKWindow::OnSize(UINT nType, int cx, int cy)
{
  int w, h;

  if (tk.Minimized && (nType==SIZE_RESTORED))
  {
    tk.Minimized = FALSE;
    return;
  }
  tk.Minimized = (nType==SIZE_MINIMIZED);
  if (tk.Minimized) return;

  ::GetWindowRect(tk.HWin,&(tk.R));
  w = tk.R.right - tk.R.left;
  h = tk.R.bottom - tk.R.top;

  (*TEKWMSize)(&tk,&ts,w,h,cx,cy);
}

void CTEKWindow::OnSysCommand(UINT nID, LPARAM lParam)
{
  if (nID==CM_SHOWMENUBAR)
  {
    ts.PopupMenu = 0;
    SwitchMenu();
  }
  else CWnd::OnSysCommand(nID,lParam);
}

void CTEKWindow::OnSysKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
  if (nChar==VK_F10)
    OnKeyDown(nChar,nRepCnt,nFlags);
  else
    CWnd::OnSysKeyDown(nChar,nRepCnt,nFlags);
}

void CTEKWindow::OnSysKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags)
{
  if (nChar==VK_F10)
    OnKeyUp(nChar,nRepCnt,nFlags);
  else
    CWnd::OnSysKeyUp(nChar,nRepCnt,nFlags);
}

LONG CTEKWindow::OnChangeMenu(UINT wParam, LONG lParam)
{
  (*TEKChangeMenu)(&tk,&ts);
  return 0;
}

LONG CTEKWindow::OnDlgHelp(UINT wParam, LONG lParam)
{
  OpenHelp(tk.HWin,HELP_CONTEXT,HelpId);
  return 0;
}

void CTEKWindow::CMFilePrint()
{
  BOOL Sel;

  HelpId = HlpTEKFilePrint;

  Sel = tk.Select;
  if (! PrnBox(tk.HWin,&Sel)) return;
  if (! PrnStart(ts.Title)) return;

  (*TEKPrint)(&tk,&ts,PrintDC,Sel);

  PrnStop();
}

void CTEKWindow::CMFileExit()
{
  DestroyWindow();
}

void CTEKWindow::CMEditCopy()
{
  (*TEKCMCopy)(&tk,&ts);
}

void CTEKWindow::CMEditCopyScreen()
{
  (*TEKCMCopyScreen)(&tk,&ts);
}

void CTEKWindow::CMEditPaste()
{
  CBStartPaste(tk.HWin,FALSE);
}

void CTEKWindow::CMEditPasteCR()
{
  CBStartPaste(tk.HWin,TRUE);
}

void CTEKWindow::CMEditClearScreen()
{
  (*TEKClearScreen)(&tk);
}

void CTEKWindow::CMSetupWindow()
{
  HelpId = HlpTEKSetupWindow;
  (*TEKSetupWinDlg)(&tk,&ts);
  if (ts.Ok)
  {
    ChangeTitle();
    SwitchMenu();
  }
}

void CTEKWindow::CMSetupFont()
{
  HelpId = HlpTEKSetupFont;
  (*TEKSetupFont)(&tk,&ts);
}

void CTEKWindow::CMVTWin()
{
  VTActivate();
}

void CTEKWindow::CMHelpIndex()
{
  OpenHelp(tk.HWin,HELP_INDEX,0);
}

void CTEKWindow::CMHelpUsing()
{
  ::WinHelp(tk.HWin,"", HELP_HELPONHELP, 0);
}

void CTEKWindow::CMAbout()
{
  if (! LoadTTDLG()) return;
  (*AboutDialog)(tk.HWin);
  FreeTTDLG();
}

