/* Tera Term
 Copyright(C) 1994-1996 T. Teranishi
 All rights reserved. */

/* TERATERM.EXE, main */

#include "teraterm.h"
#include "tttypes.h"
#include "commlib.h"
#include "ttwinman.h"
#include "vtdisp.h"
#include "vtterm.h"
#include "vtwin.h"
#include "clipboar.h"
#include "ttftypes.h"
#include "filesys.h"
#include "telnet.h"
#include "tektypes.h"
#include "tek.h"
#include "ttdde.h"

// Define an application class derived from CWinApp
class CTeraApp : public CWinApp
{
public:
  virtual BOOL InitInstance();
  virtual BOOL OnIdle(LONG lCount);
  virtual BOOL PreTranslateMessage(MSG *pMsg);
};

// Construct the CTeraApp's m_pMainWnd data member
BOOL CTeraApp::InitInstance()
{
  m_pMainWnd = new CVTWindow();
  pVTWin = m_pMainWnd;
  return TRUE;
}

// Tera Term main engine
BOOL CTeraApp::OnIdle(LONG lCount)
{
  static int Busy = 2;
  int Change;

  if (lCount==0) Busy = 2;

  if (cv.Ready)
  {
    /* Sender */
    CommSend(&cv);

    /* Parser */
    if ((cv.HLogBuf!=NULL) && (cv.LogBuf==NULL))
      cv.LogBuf = (PCHAR)GlobalLock(cv.HLogBuf);

    if ((cv.HBinBuf!=NULL) && (cv.BinBuf==NULL))
      cv.BinBuf = (PCHAR)GlobalLock(cv.HBinBuf);

    if ((TelStatus==TelIdle) && cv.TelMode)
      TelStatus = TelIAC;

    if (TelStatus != TelIdle)
    {
      ParseTel();
      if (tr.ChangeWinSize)
        ChangeTerminalSize(tr.WinSize.x,
                           tr.WinSize.y);
    }
    else {
      if (ProtoFlag) Change = ProtoDlgParse();
      else {
        switch (ActiveWin) {
          case IdVT: Change = VTParse(); break;
          case IdTEK: Change = TEKWinParse(); break;
          default:
            Change = 0;
        }
      }
      switch (Change) {
        case IdVT: VTActivate(); break;
        case IdTEK: TEKWinActivate(); break;
      }

    }

    if (cv.LogBuf!=NULL)
    {
      if (FileLog) LogToFile();
      if (DDELog && AdvFlag) DDEAdv();
      GlobalUnlock(cv.HLogBuf);
      cv.LogBuf = NULL;
    }

    if (cv.BinBuf!=NULL)
    {
      if (BinLog) LogToFile();
      GlobalUnlock(cv.HBinBuf);
      cv.BinBuf = NULL;
    }

    /* Talker */
    switch (TalkStatus) {
      case IdTalkCB: CBSend(); break; /* clip board */
      case IdTalkFile: FileSend(); break; /* file */
    }

    /* Receiver */
    CommReceive(&cv);

  }

  if (cv.Ready &&
      (cv.RRQ || (cv.OutBuffCount>0) ||
       (cv.LCount>0) || (cv.BCount>0) || (cv.DCount>0)) )
    Busy = 2;
  else
    Busy--;

  return (Busy>0);
}

BOOL CTeraApp::PreTranslateMessage(MSG *pMsg)
{
  if (ts.MetaKey>0)
    return FALSE; /* ignore accelerator keys */
  else
    return CWinApp::PreTranslateMessage(pMsg);  
}

static CTeraApp TeraApp;  // TeraApp's constructor initializes and runs the app

