/* Tera Term
 Copyright(C) 1994-1996 T. Teranishi
 All rights reserved. */

/* TTTEK.DLL, for TEK window */
#include "teraterm.h"
#include "tttypes.h"
#include "tektypes.h"
#include "ttcommon.h"
#include "ttdialog.h"
#include "tekesc.h"

static HANDLE HInst;

void FAR PASCAL TEKInit(PTEKVar tk, PTTSet ts)
{
  int i;

  if ((ts->HideTitle==0) && (ts->PopupMenu==0))
  {
    tk->MainMenu = LoadMenu(HInst, MAKEINTRESOURCE(Id_TEKMenu));
    tk->EditMenu = GetSubMenu(tk->MainMenu,1);
  }
  else
    tk->MainMenu = NULL;

  tk->MemDC = NULL;
  tk->HBits = NULL;
  tk->Pen = NULL;
  tk->MemPen = NULL;
  tk->ps = PS_SOLID;
  tk->BackGround = NULL;
  tk->MemBackGround = NULL;
  for (i = 0 ; i <= 3; i++)
    tk->TEKFont[i] = NULL;
  tk->ScreenHeight = 450;
  tk->ScreenWidth = 600;
  tk->AdjustSize = FALSE;
  tk->ScaleFont = FALSE;
  tk->TextSize = 0;

  memset(&(tk->TEKlf), 0, sizeof(LOGFONT));
  tk->TEKlf.lfHeight  = ts->TEKFontSize.y; /* Font Height */
  tk->TEKlf.lfWidth   = ts->TEKFontSize.x; /* Font Width */
  tk->TEKlf.lfCharSet = ts->TEKFontCharSet; /* Character Set */
  strcpy(tk->TEKlf.lfFaceName, ts->TEKFont);

  tk->MoveFlag = TRUE;

  tk->ParseMode = ModeFirst;
  tk->DispMode = IdAlphaMode;
  tk->Drawing = FALSE;

  tk->RubberBand = FALSE;
  tk->Select = FALSE;
  tk->ButtonDown = FALSE;

  tk->GIN = FALSE;
  tk->CrossHair = FALSE;
  tk->IgnoreCount = 0;
  tk->GINX = 0;
  tk->GINY = 0;
}

void ToggleCrossHair(PTEKVar tk, PTTSet ts, BOOL OnFlag)
{
  HDC DC;
  HPEN TempPen, OldPen;

  if (tk->CrossHair==OnFlag) return;
  DC = GetDC(tk->HWin);
  TempPen = CreatePen(PS_SOLID, 1, ts->TEKColor[0]);
  OldPen = SelectObject(DC,TempPen);
  SetROP2(DC, R2_NOT);
  MoveToEx(DC,tk->GINX,0,NULL);
  LineTo(DC,tk->GINX,tk->ScreenHeight-1);
  MoveToEx(DC,0,tk->GINY,NULL);
  LineTo(DC,tk->ScreenWidth-1,tk->GINY);
  SelectObject(DC,OldPen);
  DeleteObject(TempPen);
  ReleaseDC(tk->HWin,DC);
  tk->CrossHair = OnFlag;
}

void TEKCaretOn(PTEKVar tk, PTTSet ts)
{
  if (! tk->Active) return;
  if (tk->DispMode == IdAlphaMode)
  {
    if (ts->CursorShape==IdHCur)
      SetCaretPos(tk->CaretX,
	tk->CaretY + tk->FontHeight - CurWidth);
    else
      SetCaretPos(tk->CaretX, tk->CaretY);

    while (tk->CaretStatus > 0)
    {
      ShowCaret(tk->HWin);
      tk->CaretStatus--;
    }
  }
}

void TEKCaretOff(PTEKVar tk)
{
  if (! tk->Active) return;
  if (tk->CaretStatus == 0)
  {
    HideCaret(tk->HWin);
    tk->CaretStatus++;
  }
}

void SwitchRubberBand(PTEKVar tk, PTTSet ts, BOOL OnFlag)
{
  HDC DC;
  HPEN TempPen, OldPen;
  int OldMemRop;
  HBRUSH OldMemBrush;

  if (tk->RubberBand==OnFlag) return;

  TempPen = CreatePen(PS_DOT, 1, ts->TEKColor[0]);
  DC = GetDC(tk->HWin);
  SetBkMode(DC,1);
  SelectObject(DC,GetStockObject(HOLLOW_BRUSH));
  OldPen = SelectObject(DC,TempPen);
  SetROP2(DC, R2_NOT);
  Rectangle(DC,tk->SelectStart.x,tk->SelectStart.y,
    tk->SelectEnd.x,tk->SelectEnd.y);
  SelectObject(DC,OldPen);
  ReleaseDC(tk->HWin,DC);
  DeleteObject(TempPen);

  TempPen = CreatePen(PS_DOT, 1, tk->MemForeColor);
  OldMemBrush = SelectObject(tk->MemDC,GetStockObject(HOLLOW_BRUSH));
  SelectObject(tk->MemDC,TempPen);
  OldMemRop = SetROP2(tk->MemDC, R2_XORPEN);
  Rectangle(tk->MemDC,tk->SelectStart.x,tk->SelectStart.y,
    tk->SelectEnd.x,tk->SelectEnd.y);
  SelectObject(tk->MemDC,OldMemBrush);
  SetROP2(tk->MemDC,OldMemRop);
  SelectObject(tk->MemDC,tk->MemPen);
  DeleteObject(TempPen);     

  tk->RubberBand = OnFlag;
}

void FAR PASCAL ChangeCaret(PTEKVar tk, PTTSet ts)
{
  switch (ts->CursorShape) {
    case IdHCur:
      CreateCaret(tk->HWin, 0, tk->FontWidth, CurWidth);
      break;
    case IdVCur:
      CreateCaret(tk->HWin, 0, CurWidth, tk->FontHeight);
      break;
    default:
      CreateCaret(tk->HWin, 0, tk->FontWidth, tk->FontHeight);
  }
  tk->CaretStatus = 1;
  TEKCaretOn(tk,ts);
}

void FAR PASCAL TEKResizeWindow(PTEKVar tk, PTTSet ts, int W, int H)
{
  int i, Height, Width;
  TEXTMETRIC Metrics;
  HDC TempDC;

  if (tk->Select)
    SwitchRubberBand(tk, ts, FALSE);
  tk->Select = FALSE;

  /* Delete old MemDC */
  if (tk->MemDC != NULL)
  {
    SelectObject(tk->MemDC, tk->OldMemFont);
    SelectObject(tk->MemDC, tk->OldMemBmp);
    SelectObject(tk->MemDC, tk->OldMemPen);
    DeleteDC(tk->MemDC);
  }

  /* Delete old fonts */
  for (i =0; i <= 3; i++)
    if (tk->TEKFont[i] != NULL)
      DeleteObject(tk->TEKFont[i]);

  /* Delete old bitmap */
  if (tk->HBits != NULL) DeleteObject(tk->HBits);

  /* Delete old pen */
  if (tk->Pen != NULL) DeleteObject(tk->Pen);
  if (tk->MemPen != NULL) DeleteObject(tk->MemPen);

  /* Delete old brush */
  if (tk->BackGround != NULL) DeleteObject(tk->BackGround);
  if (tk->MemBackGround != NULL) DeleteObject(tk->MemBackGround);

  /* get DC */
  TempDC = GetDC(tk->HWin);
  tk->MemDC = CreateCompatibleDC(TempDC);

  /* Create standard size font */
  if (tk->ScaleFont)
  {
    tk->TEKlf.lfHeight = (int)((float)tk->ScreenHeight / 35.0);
    tk->TEKlf.lfWidth = (int)((float)tk->ScreenWidth / 74.0);
  }

  tk->TEKlf.lfWeight = FW_NORMAL;
  tk->TEKlf.lfItalic = 0;
  tk->TEKlf.lfUnderline = 0;
  tk->TEKlf.lfStrikeOut = 0;
  tk->TEKlf.lfOutPrecision  = OUT_CHARACTER_PRECIS;
  tk->TEKlf.lfClipPrecision = CLIP_CHARACTER_PRECIS;
  tk->TEKlf.lfQuality       = DEFAULT_QUALITY;
  tk->TEKlf.lfPitchAndFamily = FIXED_PITCH | FF_MODERN;

  tk->TEKFont[0] = CreateFontIndirect(&tk->TEKlf);
  /* Check standard font size */
  SelectObject(TempDC,tk->TEKFont[0]);
  GetTextMetrics(TempDC, &Metrics);
  tk->FW[0] = Metrics.tmAveCharWidth;
  tk->FH[0] = Metrics.tmHeight;
 
  if (! tk->ScaleFont)
  {
    tk->ScreenHeight = tk->FH[0]*35;
    tk->ScreenWidth = tk->FW[0]*74;
    Width = (int)((float)tk->ScreenHeight / 3120.0 * 4096.0);
    if (tk->ScreenWidth < Width)
      tk->ScreenWidth = Width;
  }

  Height = tk->TEKlf.lfHeight;
  Width = tk->TEKlf.lfWidth;

  /* second font */
  tk->TEKlf.lfHeight = (int)((float)tk->ScreenHeight / 38.0);
  tk->TEKlf.lfWidth = (int)((float)tk->ScreenWidth / 80.0);
  tk->TEKFont[1] = CreateFontIndirect(&tk->TEKlf);
  SelectObject(TempDC,tk->TEKFont[1]);
  GetTextMetrics(TempDC, &Metrics);
  tk->FW[1] = Metrics.tmAveCharWidth;
  tk->FH[1] = Metrics.tmHeight;

  /* third font */
  tk->TEKlf.lfHeight = (int)((float)tk->ScreenHeight / 58.0);
  tk->TEKlf.lfWidth = (int)((float)tk->ScreenWidth / 121.0);
  tk->TEKFont[2] = CreateFontIndirect(&tk->TEKlf);
  SelectObject(TempDC,tk->TEKFont[2]);
  GetTextMetrics(TempDC, &Metrics);
  tk->FW[2] = Metrics.tmAveCharWidth;
  tk->FH[2] = Metrics.tmHeight;

  /* forth font */
  tk->TEKlf.lfHeight = (int)((float)tk->ScreenHeight / 64.0);
  tk->TEKlf.lfWidth = (int)((float)tk->ScreenWidth / 133.0);
  tk->TEKFont[3] = CreateFontIndirect(&tk->TEKlf);
  SelectObject(TempDC,tk->TEKFont[3]);
  GetTextMetrics(TempDC, &Metrics);
  tk->FW[3] = Metrics.tmAveCharWidth;
  tk->FH[3] = Metrics.tmHeight;

  tk->OldMemFont =
    SelectObject(tk->MemDC,tk->TEKFont[tk->TextSize]);
  tk->FontWidth = tk->FW[tk->TextSize];
  tk->FontHeight = tk->FH[tk->TextSize];
  for (i = 0; i <= 1; i++)
    tk->Dx[i] = tk->FontWidth;

  tk->TEKlf.lfHeight = Height;
  tk->TEKlf.lfWidth = Width;

  if (ts->TEKColorEmu>0)
    tk->HBits =
      CreateCompatibleBitmap(TempDC,tk->ScreenWidth,tk->ScreenHeight);
  else
    tk->HBits =
      CreateBitmap(tk->ScreenWidth, tk->ScreenHeight, 1, 1, NULL);

  tk->OldMemBmp = SelectObject(tk->MemDC, tk->HBits);

  tk->TextColor = ts->TEKColor[0];
  if (ts->TEKColorEmu>0)
  {
    tk->MemForeColor = ts->TEKColor[0];
    tk->MemBackColor = ts->TEKColor[1];
  }
  else {
    tk->MemForeColor = RGB(0,0,0);
    tk->MemBackColor = RGB(255,255,255);
  }
  tk->MemTextColor = tk->MemForeColor;

  SetTextColor(tk->MemDC, tk->MemTextColor);
  SetBkColor(tk->MemDC, tk->MemBackColor);
  SetBkMode(tk->MemDC, 1);

  tk->BackGround = CreateSolidBrush(ts->TEKColor[1]);
  tk->MemBackGround = CreateSolidBrush(tk->MemBackColor);

  tk->PenColor = ts->TEKColor[0];
  tk->Pen = CreatePen(tk->ps,1,tk->PenColor);

  tk->MemPenColor = tk->MemForeColor;
  tk->MemPen = CreatePen(tk->ps,1,tk->MemPenColor);
  tk->OldMemPen = SelectObject(tk->MemDC, tk->MemPen);

  ReleaseDC(tk->HWin,TempDC);

  tk->R.left = 0;
  tk->R.right = tk->ScreenWidth;
  tk->R.top = 0;
  tk->R.bottom = tk->ScreenHeight;
  FillRect(tk->MemDC,&(tk->R),tk->MemBackGround);
  InvalidateRect(tk->HWin, &(tk->R), TRUE);

  tk->DispMode = IdAlphaMode;
  tk->HiY = 0;
  tk->Extra = 0;
  tk->LoY = 0;
  tk->HiX = 0;
  tk->LoX = 0;
  tk->CaretX = 0;
  tk->CaretOffset = 0;
  tk->CaretY = 0;
  if (tk->Active) ChangeCaret(tk,ts);

  GetClientRect(tk->HWin,&(tk->R));
  Width = W + tk->ScreenWidth - tk->R.right + tk->R.left;
  Height = H + tk->ScreenHeight - tk->R.bottom + tk->R.top;

  if ((Width != W) || (Height != H))
  {
    tk->AdjustSize = TRUE;
    SetWindowPos(tk->HWin,HWND_TOP,0,0,Width,Height,SWP_NOMOVE);
  }

  tk->ScaleFont = FALSE;
}

int FAR PASCAL TEKParse(PTEKVar tk, PTTSet ts, PComVar cv)
{
  BOOL f;
  int c;
  BYTE b;
  MSG DummyMsg;

  if (tk->ButtonDown)
    return 0;

  tk->ChangeEmu = 0;
  f = TRUE;

  do {
    c = CommRead1Byte(cv,&b);

    if (c > 0)
    {
      if (tk->IgnoreCount <= 0)
      {
        if (f)
        {
          TEKCaretOff(tk);
          f = FALSE;
          if (tk->GIN) ToggleCrossHair(tk,ts,FALSE);
          if (tk->RubberBand) SwitchRubberBand(tk,ts,FALSE);
        }

        switch (tk->ParseMode) {
          case ModeFirst:
	    ParseFirst(tk,ts,cv,b);
	    break;
          case ModeEscape:
	    TEKEscape(tk,ts,cv,b);
	    break;
          case ModeSelectCode:
	    SelectCode(tk,ts,b);
	    break;
          case Mode2OC:
	    TwoOpCode(tk,ts,cv,b);
	    break;
          default:
            tk->ParseMode = ModeFirst;
            ParseFirst(tk,ts,cv,b);
        }
        PeekMessage(&DummyMsg, 0, 0, 0, PM_NOREMOVE);
      }
      else tk->IgnoreCount--;
    }
  } while ((c!=0) && (tk->ChangeEmu<=0));

  ToggleCrossHair(tk,ts,tk->GIN);
  if (! f)
  {
    TEKCaretOn(tk,ts);
    SwitchRubberBand(tk,ts,tk->Select);
  }

  if (tk->ChangeEmu > 0) tk->ParseMode = ModeFirst;
  return (tk->ChangeEmu);
}

void FAR PASCAL ReportGIN(PTEKVar tk, PTTSet ts, PComVar cv, BYTE KeyCode)
{
  BYTE Code[11];
  int X, Y;

  ToggleCrossHair(tk,ts,FALSE);
  X = (int)((float)tk->GINX / (float)tk->ScreenWidth * (float)ViewSizeX);
  Y = (int)((1.0 - (float)(tk->GINY+1) / (float)tk->ScreenHeight) * (float)ViewSizeY);
  Code[0] = KeyCode;
  Code[1] = (X >> 7) + 32;
  Code[2] = ((X >> 2) & 0x1f) + 32;
  Code[3] = (Y >> 7) + 32;
  Code[4] = ((Y >> 2) & 0x1f) + 32;
  Code[5] = 0x0d;
  tk->GIN = FALSE;
  ReleaseCapture();
  CommBinaryOut(cv, &Code[0],6);
  tk->IgnoreCount = 6;
}

void FAR PASCAL TEKPaint
  (PTEKVar tk, PTTSet ts, HDC PaintDC, PAINTSTRUCT *PaintInfo)
{
  int X,Y,W,H;

  if (PaintInfo->fErase)
    FillRect(PaintDC, &(PaintInfo->rcPaint),tk->BackGround);

  if (tk->GIN) ToggleCrossHair(tk,ts,FALSE);
  if (tk->Select) SwitchRubberBand(tk,ts,FALSE);
  X = PaintInfo->rcPaint.left;
  Y = PaintInfo->rcPaint.top;
  W = PaintInfo->rcPaint.right - X;
  H = PaintInfo->rcPaint.bottom - Y;
  SetTextColor(PaintDC, ts->TEKColor[0]);
  SetBkColor(PaintDC,  ts->TEKColor[1]);
  BitBlt(PaintDC,X,Y,W,H,tk->MemDC,X,Y,SRCCOPY);
  SwitchRubberBand(tk,ts,tk->Select);
  if (tk->GIN) ToggleCrossHair(tk,ts,TRUE);
}
void FAR PASCAL TEKInitMenu(PTEKVar tk, PComVar cv, HMENU SubMenu)
{
  if (SubMenu==tk->EditMenu)
  {
    if (tk->Select)
      EnableMenuItem(tk->EditMenu,CM_TEKEDITCOPY,MF_BYCOMMAND | MF_ENABLED);
    else
      EnableMenuItem(tk->EditMenu,CM_TEKEDITCOPY,MF_BYCOMMAND | MF_GRAYED);

    if (cv->Ready &&
        (IsClipboardFormatAvailable(CF_TEXT) ||
         IsClipboardFormatAvailable(CF_OEMTEXT)))
    {
      EnableMenuItem(tk->EditMenu,CM_TEKEDITPASTE,MF_BYCOMMAND | MF_ENABLED);
      EnableMenuItem(tk->EditMenu,CM_TEKEDITPASTECR,MF_BYCOMMAND | MF_ENABLED);
    }
    else {
      EnableMenuItem(tk->EditMenu,CM_TEKEDITPASTE,MF_BYCOMMAND | MF_GRAYED);
      EnableMenuItem(tk->EditMenu,CM_TEKEDITPASTECR,MF_BYCOMMAND | MF_GRAYED);
    }
  }
}

void FAR PASCAL TEKWMLButtonDown
  (PTEKVar tk, PTTSet ts, PComVar cv, BOOL Popup, POINT pos)
{
  BYTE b;

  if (tk->GIN)
  {
    b = ts->GINMouseCode & 0xff;
    ReportGIN(tk,ts,cv,b);
    return;
  }

  if (tk->ButtonDown) return;

  if (Popup)
  {
    tk->PopupMenu = LoadMenu(HInst, MAKEINTRESOURCE(Id_TEKMenu));
    tk->EditMenu = GetSubMenu(tk->PopupMenu,1);
    TEKInitMenu(tk,cv,tk->EditMenu);
    tk->PopupBase = CreatePopupMenu();
    AppendMenu(tk->PopupBase, MF_STRING | MF_ENABLED | MF_POPUP,
               (UINT)GetSubMenu(tk->PopupMenu,0), "&File");
    AppendMenu(tk->PopupBase, MF_STRING | MF_ENABLED | MF_POPUP,
               (UINT)tk->EditMenu, "&Edit");
    AppendMenu(tk->PopupBase, MF_STRING | MF_ENABLED | MF_POPUP,
               (UINT)GetSubMenu(tk->PopupMenu,2), "&Setup");
    AppendMenu(tk->PopupBase, MF_STRING | MF_ENABLED,
               CM_TEKVTWIN, "VT&Win");
    AppendMenu(tk->PopupBase, MF_STRING | MF_ENABLED | MF_POPUP,
               (UINT)GetSubMenu(tk->PopupMenu,4), "&Help");
    return;
  }

  /* Capture mouse */
  SetCapture(tk->HWin);

  /* Is the position in client area? */
  if ((pos.x>=0) && (pos.x < tk->ScreenWidth) &&
      (pos.y>=0) && (pos.y < tk->ScreenHeight))
  {
    SwitchRubberBand(tk,ts,FALSE);
    tk->Select = FALSE;

    tk->SelectStart.x = pos.x;
    tk->SelectStart.y = pos.y;
    tk->SelectEnd = tk->SelectStart;
    tk->ButtonDown = TRUE;
  }
}

void FAR PASCAL TEKWMLButtonUp(PTEKVar tk, PTTSet ts)
{
  int X;

  ReleaseCapture();
  tk->ButtonDown = FALSE;
  if ((abs(tk->SelectEnd.y - tk->SelectStart.y) > 2) &&
      (abs(tk->SelectEnd.x - tk->SelectStart.x) > 2))
  {
    if (tk->SelectStart.x > tk->SelectEnd.x)
    {
      X = tk->SelectEnd.x;
      tk->SelectEnd.x = tk->SelectStart.x; 
      tk->SelectStart.x = X;
    }
    if (tk->SelectStart.y > tk->SelectEnd.y)
    {
      X = tk->SelectEnd.y;
      tk->SelectEnd.y = tk->SelectStart.y; 
      tk->SelectStart.y = X;
    }
    tk->Select = TRUE;
  }
  else {
    SwitchRubberBand(tk,ts,FALSE);
    tk->Select = FALSE;
  }
}

void FAR PASCAL TEKWMMouseMove(PTEKVar tk, PTTSet ts, POINT p)
{
  int X, Y;

  if ((! tk->ButtonDown) && (! tk->GIN)) return;
  /* get position */
  X = p.x + 1;
  Y = p.y + 1;

  /* if out of client area, force into client area */
  if (X<0) X = 0;
  else if (X > tk->ScreenWidth) X = tk->ScreenWidth - 1;
  if (Y<0) Y = 0;
  else if (Y > tk->ScreenHeight) Y = tk->ScreenHeight - 1;

  if (tk->GIN)
  {
    ToggleCrossHair(tk,ts,FALSE);
    tk->GINX = X;
    tk->GINY = Y;
    ToggleCrossHair(tk,ts,TRUE);
  }
  else {
    SwitchRubberBand(tk,ts,FALSE);
    tk->SelectEnd.x = X + 1;
    tk->SelectEnd.y = Y + 1;
    SwitchRubberBand(tk,ts,TRUE);
  }

  if (tk->GIN) SetCapture(tk->HWin);
}

void FAR PASCAL TEKWMSize(PTEKVar tk, PTTSet ts, int W, int H, int cx, int cy)
{
  int Width, Height;

  Width = cx;
  Height = cy;

  if ((tk->ScreenWidth == Width) &&
      (tk->ScreenHeight == Height))
  {
    tk->AdjustSize = FALSE;
    return;
  }

  if (tk->AdjustSize)
  {
    Width = W + tk->ScreenWidth - Width;
    Height = H + tk->ScreenHeight - Height;
    SetWindowPos(tk->HWin,HWND_TOP,0,0,Width,Height,SWP_NOMOVE);
  }
  else {
    tk->ScreenWidth = Width;
    tk->ScreenHeight = Height;
    tk->ScaleFont = TRUE;
    TEKResizeWindow(tk,ts,W,H);
  }
}

void CopyToClipboard
  (PTEKVar tk, PTTSet ts, int x, int y, int Width, int Height)
{
  HDC CopyDC;
  HBITMAP CopyBitmap;

  if (tk->Select) SwitchRubberBand(tk,ts,FALSE);
  TEKCaretOff(tk);
  if (OpenClipboard(tk->HWin) && EmptyClipboard())
  {
    /* Create the new bitmap */
    CopyDC = CreateCompatibleDC(tk->MemDC);
    CopyBitmap = CreateCompatibleBitmap(tk->MemDC, Width, Height);
    CopyBitmap = SelectObject(CopyDC, CopyBitmap);
    BitBlt(CopyDC, 0, 0, Width, Height, tk->MemDC, x, y, SRCCOPY);
    CopyBitmap = SelectObject(CopyDC, CopyBitmap);
    /* Transfer the new bitmap to the clipboard */
    SetClipboardData(CF_BITMAP, CopyBitmap);
  }

  CloseClipboard();
  DeleteDC(CopyDC);

  TEKCaretOn(tk,ts);
  SwitchRubberBand(tk,ts,tk->Select);
}

void FAR PASCAL TEKCMCopy(PTEKVar tk, PTTSet ts)
{
  int x, y;

  if (! tk->Select) return;

  if (tk->SelectStart.x < tk->SelectEnd.x)
    x = tk->SelectStart.x;
  else x = tk->SelectEnd.x;
  if (tk->SelectStart.y < tk->SelectEnd.y)
    y = tk->SelectStart.y;
  else y = tk->SelectEnd.y;
  /* copy selected area to clipboard */
  CopyToClipboard(tk, ts, x,y,
    abs(tk->SelectEnd.x - tk->SelectStart.x),
    abs(tk->SelectEnd.y - tk->SelectStart.y));                          
}

void FAR PASCAL TEKCMCopyScreen(PTEKVar tk, PTTSet ts)
{
  /* copy fullscreen to clipboard */
  CopyToClipboard(tk, ts, 0, 0, tk->ScreenWidth, tk->ScreenHeight);
}

void FAR PASCAL TEKPrint(PTEKVar tk, PTTSet ts, HDC PrintDC, BOOL SelFlag)
{
  POINT PPI;
  RECT Mergin;
  int Caps;
  int PrnWidth, PrnHeight;
  int MemWidth, MemHeight;
  POINT PrintStart, PrintEnd;

  if (PrintDC == NULL) return;

  if (SelFlag)
  {
    /* print selection */
    PrintStart = tk->SelectStart;
    PrintEnd = tk->SelectEnd;
  }
  else {
    /* print current page */
    PrintStart.x = 0;
    PrintStart.y = 0;
    PrintEnd.x = tk->ScreenWidth;
    PrintEnd.y = tk->ScreenHeight;
  }

  Caps = GetDeviceCaps(PrintDC,RASTERCAPS);
  if ((Caps & RC_BITBLT) != RC_BITBLT)
  {
    MessageBox(tk->HWin,"Printer dose not support graphics",
      "Tera Term: Error",MB_ICONEXCLAMATION);
    return;
  }
  if (tk->Active) TEKCaretOff(tk);
  if (tk->RubberBand) SwitchRubberBand(tk,ts,FALSE);

  MemWidth = PrintEnd.x - PrintStart.x;
  MemHeight = PrintEnd.y - PrintStart.y;
  if ((MemWidth==0) || (MemHeight==0)) return;

  StartPage(PrintDC);

  PPI.x = GetDeviceCaps(PrintDC,LOGPIXELSX);
  PPI.y = GetDeviceCaps(PrintDC,LOGPIXELSY);

  /* horizontal mergin: about 1/4 inch */
  Mergin.left = (int)((float)PPI.x * 0.25);
  Mergin.right = GetDeviceCaps(PrintDC,HORZRES) - Mergin.left;
  /* virtical mergin: about 1 inch */
  Mergin.top = (int)((float)PPI.y * 1.0);
  Mergin.bottom = GetDeviceCaps(PrintDC,VERTRES) - Mergin.top;

  if ((Caps & RC_STRETCHBLT) == RC_STRETCHBLT)
  {
    PrnWidth = (int)((float)MemWidth /
      (float)GetDeviceCaps(tk->MemDC,LOGPIXELSX) * (float)PPI.x);
    PrnHeight = (int)((float)MemHeight /
      (float)GetDeviceCaps(tk->MemDC,LOGPIXELSY) * (float)PPI.y);
    StretchBlt(PrintDC, Mergin.left, Mergin.top, PrnWidth, PrnHeight,
               tk->MemDC, PrintStart.x, PrintStart.y, MemWidth, MemHeight, SRCCOPY);
  }
  else
    BitBlt(PrintDC, Mergin.left, Mergin.top, MemWidth, MemHeight,
               tk->MemDC, PrintStart.x, PrintStart.y, SRCCOPY);

  EndPage(PrintDC);

  SwitchRubberBand(tk,ts,tk->Select);
}

void FAR PASCAL TEKClearScreen(PTEKVar tk)
{
  GetClientRect(tk->HWin,&(tk->R));
  FillRect(tk->MemDC,&(tk->R),tk->MemBackGround);
  InvalidateRect(tk->HWin,&(tk->R),FALSE);
  tk->DispMode = IdAlphaMode;
  tk->CaretX = 0;
  tk->CaretOffset = 0;
  tk->CaretY = 0;
  if (tk->Active) SetCaretPos(tk->CaretX,tk->CaretY);
}

void FAR PASCAL TEKSetupFont(PTEKVar tk, PTTSet ts)
{
  int W, H;
  BOOL Ok;

  if (! LoadTTDLG()) return;
  Ok = (*ChooseFontDlg)(tk->HWin,&tk->TEKlf,NULL);
  FreeTTDLG();
  if (! Ok) return;

  strcpy(ts->TEKFont,tk->TEKlf.lfFaceName);
  ts->TEKFontSize.x = tk->TEKlf.lfWidth;
  ts->TEKFontSize.y = tk->TEKlf.lfHeight;
  ts->TEKFontCharSet = tk->TEKlf.lfCharSet;

  GetWindowRect(tk->HWin, &(tk->R));
  W = tk->R.right - tk->R.left;
  H = tk->R.bottom - tk->R.top;
  tk->TextSize = 0;
  TEKResizeWindow(tk,ts,W,H);
}

void TEKResetWin(PTEKVar tk, PTTSet ts, WORD EmuOld)
{
  HDC TmpDC;

  /* Change caret shape */
  ChangeCaret(tk,ts);

  /* Change display color*/
  TmpDC = GetDC(tk->HWin);

  if (ts->TEKColorEmu != EmuOld)
  {
    SelectObject(tk->MemDC,tk->OldMemBmp);
    if (tk->HBits!=NULL) DeleteObject(tk->HBits);
    if (ts->TEKColorEmu > 0)
      tk->HBits =
	CreateCompatibleBitmap(TmpDC,tk->ScreenWidth,tk->ScreenHeight);
    else
      tk->HBits =
	CreateBitmap(tk->ScreenWidth, tk->ScreenHeight, 1, 1, NULL);
    tk->OldMemBmp = SelectObject(tk->MemDC, tk->HBits);
    GetClientRect(tk->HWin,&(tk->R));
    FillRect(tk->MemDC,&(tk->R),tk->MemBackGround);
  }

  ReleaseDC(tk->HWin,TmpDC);

  tk->TextColor = ts->TEKColor[0];

  tk->ps = PS_SOLID;
  tk->PenColor = ts->TEKColor[0];
  if (tk->Pen!=NULL) DeleteObject(tk->Pen);
  tk->Pen = CreatePen(tk->ps,1,tk->PenColor);

  if (tk->BackGround!=NULL) DeleteObject(tk->BackGround);
  tk->BackGround = CreateSolidBrush(ts->TEKColor[1]);

  if (ts->TEKColorEmu>0)
  {
    tk->MemForeColor = ts->TEKColor[0];
    tk->MemBackColor = ts->TEKColor[1];
  }
  else {
    tk->MemForeColor = RGB(0,0,0);
    tk->MemBackColor = RGB(255,255,255);
  }
  tk->MemTextColor = tk->MemForeColor;
  tk->MemPenColor = tk->MemForeColor;

  SelectObject(tk->MemDC, tk->OldMemPen);
  if (tk->MemPen!=NULL) DeleteObject(tk->MemPen);
  tk->MemPen = CreatePen(tk->ps,1,tk->MemPenColor);
  tk->OldMemPen = SelectObject(tk->MemDC,tk->MemPen);

  if (tk->MemBackGround!=NULL) DeleteObject(tk->MemBackGround);
  tk->MemBackGround = CreateSolidBrush(tk->MemBackColor);

  SetTextColor(tk->MemDC, tk->MemTextColor);
  SetBkColor(tk->MemDC, tk->MemBackColor);

  if ((ts->TEKColorEmu>0) ||
      (ts->TEKColorEmu!=EmuOld))
    TEKClearScreen(tk);

  InvalidateRect(tk->HWin,NULL,TRUE);
}

void FAR PASCAL TEKSetupWinDlg(PTEKVar tk, PTTSet ts)
{
  WORD OldEmu;

  ts->Ok = FALSE;
  ts->VTFlag = FALSE;
  ts->SampleFont = tk->TEKFont[0];

  if (! LoadTTDLG()) return;
  OldEmu = ts->TEKColorEmu;
  (*SetupWin)(tk->HWin, ts);
  FreeTTDLG();

  if (ts->Ok) TEKResetWin(tk,ts,OldEmu);
}

void FAR PASCAL TEKRestoreSetup(PTEKVar tk, PTTSet ts)
{
  int W, H;

  /* change window */
  strcpy(tk->TEKlf.lfFaceName,ts->TEKFont);
  tk->TEKlf.lfWidth = ts->TEKFontSize.x;
  tk->TEKlf.lfHeight = ts->TEKFontSize.y;
  tk->TEKlf.lfCharSet = ts->TEKFontCharSet;
  tk->TextSize = 0;

  GetWindowRect(tk->HWin, &(tk->R));
  W = tk->R.right - tk->R.left;
  H = tk->R.bottom - tk->R.top;
  TEKResizeWindow(tk,ts,W,H);
}

void FAR PASCAL TEKEnd(PTEKVar tk)
{
  int i;

  if (tk->MemDC != NULL) DeleteDC(tk->MemDC);
  for (i =0; i <= 3; i++)
    if (tk->TEKFont[i]!=NULL) DeleteObject(tk->TEKFont[i]);
  if (tk->HBits != NULL) DeleteObject(tk->HBits);
  if (tk->Pen != NULL) DeleteObject(tk->Pen);
  if (tk->MemPen != NULL) DeleteObject(tk->MemPen);
  if (tk->BackGround != NULL) DeleteObject(tk->BackGround);
  if (tk->MemBackGround != NULL) DeleteObject(tk->MemBackGround);
}

void FAR PASCAL TEKChangeMenu(PTEKVar tk, PTTSet ts)
{
  HMENU SysMenu;

  if ((ts->PopupMenu > 0) && (tk->MainMenu != NULL) ||
      (ts->PopupMenu == 0) && (tk->MainMenu == NULL))
  {
    if (ts->PopupMenu>0)
    {
      DestroyMenu(tk->MainMenu);
      tk->MainMenu = NULL;
    }
    else {
      tk->MainMenu = LoadMenu(HInst, MAKEINTRESOURCE(Id_TEKMenu));
      tk->EditMenu = GetSubMenu(tk->MainMenu,1);
    }
    tk->AdjustSize = TRUE;
    SetMenu(tk->HWin, tk->MainMenu);
    DrawMenuBar(tk->HWin);
  }

  GetSystemMenu(tk->HWin,TRUE);
  if (ts->PopupMenu>0)
  {
    SysMenu = GetSystemMenu(tk->HWin,FALSE);
    AppendMenu(SysMenu, MF_SEPARATOR, 0, NULL);
    AppendMenu(SysMenu, MF_STRING, CM_SHOWMENUBAR, "Show menu &bar");
  }
}

#pragma off (unreferenced);
int APIENTRY LibMain(HANDLE hdll,
                     DWORD  reason,
                     LPVOID reserved)
#pragma on (unreferenced);
{
  HInst = hdll;
  switch( reason ) { 
  case DLL_THREAD_ATTACH:
     /* do thread initialization */
    break;
  case DLL_THREAD_DETACH:
    /* do thread cleanup */
    break;
  case DLL_PROCESS_ATTACH:
     /* do process initialization */
    break;
  case DLL_PROCESS_DETACH:
    /* do process cleanup */
    break;
  }
   return( 1 );
 }

