/* Tera Term
 Copyright(C) 1994-1996 T. Teranishi
 All rights reserved. */

/* Constants and types for Tera Term */

#define MAXPATHLEN 256

#define IdBreakTimer 1
#define IdDelayTimer 2
#define IdProtoTimer 3
#define IdDblClkTimer 4
#define IdScrollTimer 5
#define IdComEndTimer 6

  /* Window Id */
#define IdVT 1
#define IdTEK 2

  /* Talker mode */
#define IdTalkKeyb 0
#define IdTalkCB 1
#define IdTalkFile 2
#define IdTalkQuiet 3

  /* Character sets */
#define IdASCII 0
#define IdKatakana 1
#define IdKanji 2
#define IdSpecial 3

  /* Character attribute bit masks */
#define AttrDefault 0x00
#define AttrDefault2 0x00
#define AttrBold 0x01
#define AttrUnder 0x02
#define AttrSpecial 0x04
#define AttrFontMask 0x07
#define AttrBlink 0x08
#define AttrReverse 0x10
#define AttrKanji 0x80
  /* Color attribute bit masks */
#define Attr2Fore 0x08
#define Attr2ForeMask 0x07
#define Attr2Back 0x80
#define Attr2BackMask 0x70
#define SftAttrBack 4

  /* Color codes */
#define IdBack   0
#define IdRed    1
#define IdGreen  2
#define IdYellow 3
#define IdBlue   4
#define IdMagenta 5
#define IdCyan   6
#define IdFore   7

  /* Kermit function id */
#define IdKmtReceive 1
#define IdKmtGet 2
#define IdKmtSend 3
#define IdKmtFinish 4

  /* XMODEM function id */
#define IdXReceive 1
#define IdXSend 2

  /* ZMODEM function id */
#define IdZReceive 1
#define IdZSend 2
#define IdZAuto 3

  /* B-Plus function id */
#define IdBPReceive 1
#define IdBPSend 2
#define IdBPAuto 3

  /* Quick-VAN function id */
#define IdQVReceive 1
#define IdQVSend 2

#define HostNameMaxLength 80


  /* internal WM_User messages */
#define MsgAccelCommand 1
#define MsgChangeMenu 2
#define MsgCloseIME 3
#define MsgComNotify 4
#define MsgComOpen 5
#define MsgComStart 6
#define MsgDlgHelp2 7
#define MsgGetHost 8

#define MsgDdeReady 21
#define MsgDdeCmndEnd 22
#define MsgDdeComReady 23
#define MsgDdeEnd 24

  /* port type ID */
#define IdTCPIP 1
#define IdSerial 2
#define IdFile  3

  /* XMODEM option */
#define XoptCheck 1
#define XoptCRC  2
#define Xopt1K   3

  /* Language */
#define IdEnglish 1
#define IdJapanese 2

/* TTTSet */
  typedef struct {
/*------ VTSet --------*/
    BOOL Ok;  /* set by TTSET.DLL */

  /* Tera Term home directory */
  char HomeDir[MAXPATHLEN];

  /* Setup file name */
  char SetupFName[MAXPATHLEN];
  char KeyCnfFN[MAXPATHLEN];
  char HostName[81];

    POINT VTPos;
    char VTFont[LF_FACESIZE];
    POINT VTFontSize;
    int VTFontCharSet;
    int FontDW, FontDH, FontDX, FontDY;
    char PrnFont[LF_FACESIZE];
    POINT PrnFontSize;
    int PrnFontCharSet;
    WORD EnableBold;
    int ScrollThreshold;
    BOOL CtrlInKanji;
    BOOL Debug, TelLog, KmtLog, XmodemLog,
      ZmodemLog, ZmodemEscCtl, ZmodemAuto,
      BPLog, QVLog;
    int ZmodemDataLen, ZmodemWinSize;
    int QVWinSize;
    WORD TransBin;
    WORD XmodemOpt, XmodemBin;
    WORD Append;
    char FileDir[MAXPATHLEN];
    WORD Language;
/*------ TEKSet --------*/
    POINT TEKPos;
    char TEKFont[LF_FACESIZE];
    POINT TEKFontSize;
    int TEKFontCharSet;
    int GINMouseCode;
/*------ TermSet --------*/
    int TerminalWidth;
    int TerminalHeight;
    WORD TermIsWin;
    WORD AutoWinResize;
    WORD CRSend;
    WORD CRReceive;
    WORD LocalEcho;
    char Answerback[31];
    int AnswerbackLen;
    WORD KanjiCode;
    WORD KanjiCodeSend;
    WORD JIS7Katakana;
    WORD JIS7KatakanaSend;
    WORD KanjiIn;
    WORD KanjiOut;
    WORD AutoWinSwitch;
    WORD TerminalID;
    BOOL AllowWrongSequence;
    BOOL Accept8bitCtrl;
    BOOL BPAuto;  /* B-plus auto activation */
/*------ WinSet --------*/
    BOOL VTFlag;
    HFONT SampleFont;
    WORD TmpColor[3] [6];
    /* Tera Term window setup variables */
    char Title[50];
    WORD CursorShape;
    WORD EnableScrollBuff;
    WORD ScrollBuffMax;
    WORD HideTitle;
    WORD PopupMenu;
    WORD TEKColorEmu;
    COLORREF VTColor[2];
    COLORREF TEKColor[2];
    COLORREF VTBoldColor[2];
    COLORREF VTBlinkColor[2];
/*------ KeybSet --------*/
    WORD BSKey;
    WORD UseIME;
    WORD IMEInline;
    WORD MetaKey;
/*------ PortSet --------*/
    WORD PortType;
    /* TCP/IP */
    WORD TCPPort;
    WORD Telnet;
    WORD TelBin;
    char TermType[41];
    WORD AutoWinClose;
    WORD ConfirmDisconn;
    /* Serial */
    WORD ComPort;
    WORD Baud;
    WORD Parity;
    WORD DataBit;
    WORD StopBit;
    WORD Flow;
    WORD DelayPerChar;
    WORD DelayPerLine;
  } TTTSet;
typedef TTTSet *PTTSet;

  /* New Line modes */
#define IdCR 1
#define IdCRLF 2
  /* Terminal ID */
#define IdVT100 1
#define IdVT100J 2
#define IdVT101 3
#define IdVT102 4
#define IdVT102J 5
#define IdVT220J 6
#define IdVT282 7
#define IdVT320 8
#define IdVT382 9
  /* Kanji Code ID */
#define IdSJIS 1
#define IdEUC 2
#define IdJIS 3

  /* KanjiIn modes */
#define IdKanjiInA 1
#define IdKanjiInB 2
  /* KanjiOut modes */
#define IdKanjiOutB 1
#define IdKanjiOutJ 2
#define IdKanjiOutH 3

#define TermWidthMax 300
#define TermHeightMax 200

  /* Cursor shapes */
#define IdBlkCur 1
#define IdVCur 2
#define IdHCur 3

#define IdBS 1
#define IdDEL 2

  /* Serial port ID */
#define IdCOM1 1
#define IdCOM2 2
#define IdCOM3 3
#define IdCOM4 4
  /* Baud rate ID */
#define IdBaud110 1
#define IdBaud300 2
#define IdBaud600 3
#define IdBaud1200 4
#define IdBaud2400 5
#define IdBaud4800 6
#define IdBaud9600 7
#define IdBaud19200 8
#define IdBaud38400 9
#define IdBaud57600 10
#define IdBaud115200 11

  /* Parity ID */
#define IdParityEven 1
#define IdParityOdd 2
#define IdParityNone 3
  /* Data bit ID */
#define IdDataBit7 1
#define IdDataBit8 2
  /* Stop bit ID */
#define IdStopBit1 1
#define IdStopBit2 2
  /* Flow control ID */
#define IdFlowX 1
#define IdFlowHard 2
#define IdFlowNone 3

/* GetHostName dialog record */
  typedef struct {
    PCHAR SetupFN; /* setup file name */
    WORD PortType; /* TCPIP/Serial */
    PCHAR HostName; /* host name */
    WORD Telnet; /* non-zero: enable telnet */
    WORD TCPPort;
    WORD ComPort;
  }TGetHNRec;
typedef TGetHNRec *PGetHNRec;

/* Tera Term internal key codes */
#define IdUp     1
#define IdDown   2
#define IdRight  3
#define IdLeft   4
#define Id0      5
#define Id1      6
#define Id2      7
#define Id3      8
#define Id4      9
#define Id5     10
#define Id6     11
#define Id7     12
#define Id8     13
#define Id9     14
#define IdMinus 15
#define IdComma 16
#define IdPeriod 17
#define IdEnter 18
#define IdPF1   19
#define IdPF2   20
#define IdPF3   21
#define IdPF4   22
#define IdFind  23
#define IdInsert 24
#define IdRemove 25
#define IdSelect 26
#define IdPrev  27
#define IdNext  28
#define IdF6    29
#define IdF7    30
#define IdF8    31
#define IdF9    32
#define IdF10   33
#define IdF11   34
#define IdF12   35
#define IdF13   36
#define IdF14   37
#define IdHelp  38
#define IdDo    39
#define IdF17   40
#define IdF18   41
#define IdF19   42
#define IdF20   43
#define IdAccEditCopy 44
#define IdAccEditPaste 45
#define IdAccEditPasteCR 46
#define IdEditPasteCR230 /* Menu ID */
#define IdAccLineUp 47
#define IdAccLineDown 48
#define IdAccPageUp 49
#define IdAccPageDown 50
#define IdAccBuffTop 51
#define IdAccBuffBottom 52
#define IdAccSendBreak 53
#define IdUser1 54
#define NumOfUserKey 99
#define IdKeyMax IdUser1+NumOfUserKey-1

#define KeyStrMax 1023

  typedef struct {
    WORD Map[IdKeyMax];
    BYTE UserKeyStr[KeyStrMax+1];
    /* user key conv flag */
    BYTE UserKeyConv[NumOfUserKey];
    /* user key str position/length in buffer */
    int UserKeyPtr[NumOfUserKey], UserKeyLen[NumOfUserKey];
  }TKeyMap;
typedef TKeyMap *PKeyMap;

/* Control Characters */

#define NUL 0x00
#define SOH 0x01
#define STX 0x02
#define ETX 0x03
#define EOT 0x04
#define ENQ 0x05
#define ACK 0x06
#define BEL 0x07
#define BS  0x08
#define HT  0x09
#define LF  0x0A
#define VT  0x0B
#define FF  0x0C
#define CR  0x0D
#define SO  0x0E
#define SI  0x0F
#define DLE 0x10
#define DC1 0x11
	#define XON 0x11
#define DC2 0x12
#define DC3 0x13
	#define XOFF 0x13
#define DC4 0x14
#define NAK 0x15
#define SYN 0x16
#define ETB 0x17
#define CAN 0x18
#define EM  0x19
#define SUB 0x1A
#define ESC 0x1B
#define FS  0x1C
#define GS  0x1D
#define RS  0x1E
#define US  0x1F

#define SP  0x20

#define DEL 0x7F

#define IND 0x84
#define NEL 0x85
#define SSA 0x86
#define ESA 0x87
#define HTS 0x88
#define HTJ 0x89
#define VTS 0x8A
#define PLD 0x8B
#define PLU 0x8C
#define RI  0x8D
#define SS2 0x8E
#define SS3 0x8F
#define DCS 0x90
#define PU1 0x91
#define PU2 0x92
#define STS 0x93
#define CCH 0x94
#define MW  0x95
#define SPA 0x96
#define EPA 0x97
#define SOS 0x98


#define CSI 0x9B
#define ST  0x9C
#define OSC 0x9D
#define PM  0x9E
#define APC 0x9F

/* help file names */
#define HelpEng "ttermp.hlp"
#define HelpJpn "ttermpj.hlp"

/* help context numbers */
#define HlpFileNewConnection 1
#define HlpFileLog 2
#define HlpFileSend 3
#define HlpFileKmtGet 4
#define HlpFileKmtSend 5
#define HlpFileXmodemRecv 6
#define HlpFileXmodemSend 7
#define HlpFileZmodemSend 8
#define HlpFileBPlusSend 9
#define HlpFileQVSend 10
#define HlpFileChangeDir 11
#define HlpFilePrint 12
#define HlpSetupTerminal 13
#define HlpSetupWindow 14
#define HlpSetupFont 15
#define HlpSetupKeyboard 16
#define HlpSetupSerialPort 17
#define HlpSetupTCPIP 18
#define HlpSetupGeneral 19
#define HlpSetupSave 20
#define HlpSetupRestore 21
#define HlpTEKFilePrint 22
#define HlpTEKSetupWindow 23
#define HlpTEKSetupFont 24

#define InBuffSize 1024
#define OutBuffSize 1024

  typedef struct {
    BYTE InBuff[InBuffSize];
    int InBuffCount,InPtr;
    BYTE OutBuff[OutBuffSize];
    int OutBuffCount,OutPtr;

    HWND HWin;
    BOOL Ready;
    BOOL Open;
    WORD PortType;
    WORD ComPort;
    /* saddr: sockaddr_in */
    WORD s; /* SOCKET */
    WORD RetryCount;
    HANDLE ComID;

    BOOL CanSend, RRQ;

    BOOL SendKanjiFlag;
    BYTE SendKanjiFirst;
    int SendCode;
    BOOL EchoKanjiFlag;
    BYTE EchoKanjiFirst;
    int EchoCode;

    /* from VTSet */
    WORD Language;
    /* from TermSet */
    WORD CRSend;
    WORD KanjiCodeEcho;
    WORD JIS7KatakanaEcho;
    WORD KanjiCodeSend;
    WORD JIS7KatakanaSend;
    WORD KanjiIn;
    WORD KanjiOut;
    /* from PortSet */
    WORD DelayPerChar;
    WORD DelayPerLine;
    BOOL TelBinRecv, TelBinSend;

    BOOL DelayFlag;
    BOOL TelFlag, TelMode;
    BOOL IACFlag, TelCRFlag;
    BOOL TelCRSend, TelCRSendEcho;

    /* Text log */
    HANDLE HLogBuf;
    PCHAR LogBuf;
    int LogPtr, LStart, LCount;
    /* Binary log & DDE */
    HANDLE HBinBuf;
    PCHAR BinBuf;
    int BinPtr, BStart, BCount, DStart, DCount;
    int BinSkip;
    BOOL FilePause;
  }TComVar;
typedef TComVar *PComVar;

/* VT window menu ID's */
#define CM_FILE 0
#define CM_FILENEWCONNECTION 110
#define CM_FILELOG 120
#define CM_FILESEND 130
#define CM_TRANSFER 4
#define CM_FILEKERMITRCV 141
#define CM_FILEKERMITGET 142
#define CM_FILEKERMITSEND 143
#define CM_FILEKERMITFINISH 144
#define CM_FILEXRCV 145
#define CM_FILEXSEND 146
#define CM_FILEZRCV 151
#define CM_FILEZSEND 152
#define CM_FILEBPRCV 153
#define CM_FILEBPSEND 154
#define CM_FILEQVRCV 155
#define CM_FILEQVSEND 156
#define CM_FILECHANGEDIR 170
#define CM_FILEPRINT 180
#define CM_FILEDISCONNECT 190
#define CM_FILEEXIT 24340

#define CM_EDIT 1
#define CM_EDITCLEARSCREEN 240
#define CM_EDITCLEARBUFFER 250
#define CM_EDITCOPY 24321
#define CM_EDITPASTE 24322
#define CM_EDITPASTECR 230
#define CM_SETUP 2
#define CM_SETUPTERMINAL 310
#define CM_SETUPWINDOW 320
#define CM_SETUPFONT 330
#define CM_SETUPKEYBOARD 340
#define CM_SETUPSERIALPORT 350
#define CM_SETUPTCPIP 360
#define CM_SETUPGENERAL 370
#define CM_SETUPSAVE 380
#define CM_SETUPRESTORE 390

#define CM_CONTROL 3
#define CM_CONTROLRESETTERMINAL 410
#define CM_CONTROLAREYOUTHERE 420
#define CM_CONTROLSENDBREAK 430
#define CM_CONTROLRESETPORT 440
#define CM_CONTROLOPENTEK 450
#define CM_CONTROLCLOSETEK 460
#define CM_CONTROLMACRO 470

#define CM_HELPINDEX 910
#define CM_HELPUSING 920
#define CM_HELPABOUT 990

#define CM_SHOWMENUBAR 995

