/* Tera Term
 Copyright(C) 1994-1996 T. Teranishi
 All rights reserved. */

/* TTFILE.DLL, Print routine of VT window */
#include "teraterm.h"
#include "tttypes.h"
#include "ttftypes.h"

LONG GetLinePtr(PVTPrnVar vpv, int Line)
{
  LONG Ptr;

  Ptr = (LONG)(vpv->BuffStartAbs + Line) *
        (LONG)(vpv->NumOfColumns);
  if ( Ptr >= vpv->BufferSize )
    Ptr = Ptr - vpv->BufferSize;
  return Ptr;
}

LONG NextLinePtr(PVTPrnVar vpv, LONG Ptr)
{
  Ptr = Ptr + (LONG)vpv->NumOfColumns;
  if ( Ptr >= vpv->BufferSize )
    Ptr = Ptr - vpv->BufferSize;
  return Ptr;
}

void FAR PASCAL VTPrint(PVTPrnVar vpv)
{
  LOGFONT Prnlf;
  HFONT PrnFont[AttrFontMask+1];
  TEXTMETRIC Metrics;
  int FW, FH;
  RECT Mergin;
  POINT PPI, PPI2;
  COLORREF White, Black;
  int i, j, count;
  int IStart, IEnd;
  int X, Y;
  LONG TmpPtr;
  BYTE CurAttr, TempAttr;
  BYTE CurAttr2, TempAttr2;
  RECT RText;
  int Dx[256];
  HDC DC;

  if (vpv->PrintDC == 0) return;
  if ((vpv->PrintStart.y == vpv->PrintEnd.y) &&
      (vpv->PrintStart.x == vpv->PrintEnd.x)) return;

  StartPage(vpv->PrintDC);

  DC = GetDC(vpv->HWin);

  memset(&Prnlf, 0, sizeof(LOGFONT));

  if (strlen(vpv->ts->PrnFont)==0)
  {
    Prnlf.lfHeight = vpv->ts->VTFontSize.y;
    Prnlf.lfWidth = vpv->ts->VTFontSize.x;
    Prnlf.lfCharSet = vpv->ts->VTFontCharSet;
    strcpy(Prnlf.lfFaceName, vpv->ts->VTFont);
  }
  else {
    Prnlf.lfHeight = vpv->ts->PrnFontSize.y;
    Prnlf.lfWidth = vpv->ts->PrnFontSize.x;
    Prnlf.lfCharSet = vpv->ts->PrnFontCharSet;
    strcpy(Prnlf.lfFaceName, vpv->ts->PrnFont);
  }

  /* create test font */
  Prnlf.lfWeight = FW_NORMAL;
  Prnlf.lfUnderline = 0;
  PrnFont[0] = CreateFontIndirect(&Prnlf);

  SelectObject(DC, PrnFont[0]);
  GetTextMetrics(DC, &Metrics);

  /* Adjust font size */
  Prnlf.lfHeight = Metrics.tmHeight;
  Prnlf.lfWidth = Metrics.tmAveCharWidth;
    /* pixels per inch */
  PPI.x = GetDeviceCaps(vpv->PrintDC,LOGPIXELSX);
  PPI.y = GetDeviceCaps(vpv->PrintDC,LOGPIXELSY);
  PPI2.x = GetDeviceCaps(DC,LOGPIXELSX);
  PPI2.y = GetDeviceCaps(DC,LOGPIXELSY);

  Prnlf.lfHeight =
    (int)((float)Prnlf.lfHeight * (float)PPI.y / (float)PPI2.y);
  Prnlf.lfWidth =
    (int)((float)Prnlf.lfWidth * (float)PPI.x / (float)PPI2.x);

  Mergin.left =
    (int)((float)PPI.x * 0.25); /* horizontal mergin: about 1/4 inch */
  Mergin.right = GetDeviceCaps(vpv->PrintDC,HORZRES) - Mergin.left;
  Mergin.top =
    (int)((float)PPI.y * 1.0); /* virtical mergin: about 1 inch */
  Mergin.bottom = GetDeviceCaps(vpv->PrintDC,VERTRES) - Mergin.top;

  ReleaseDC(vpv->HWin,DC);
  DeleteObject(PrnFont[0]); /* Delete test font */

  /* Create New Fonts */

  /* Normal Font */
  Prnlf.lfWeight = FW_NORMAL;
  Prnlf.lfUnderline = 0;
  PrnFont[0] = CreateFontIndirect(&Prnlf);
  SelectObject(vpv->PrintDC,PrnFont[0]);
  GetTextMetrics(vpv->PrintDC, &Metrics);
  FW = Metrics.tmAveCharWidth;
  FH = Metrics.tmHeight;
  /* Under line */
  Prnlf.lfUnderline = 1;
  PrnFont[AttrUnder] = CreateFontIndirect(&Prnlf);

  if (vpv->ts->EnableBold > 0)
  {
    /* Bold */
    Prnlf.lfUnderline = 0;
    Prnlf.lfWeight = FW_BOLD;
    PrnFont[AttrBold] = CreateFontIndirect(&Prnlf);
    /* Bold + Underline */
    Prnlf.lfUnderline = 1;
    PrnFont[AttrBold | AttrUnder] = CreateFontIndirect(&Prnlf);
  }
  else {
    PrnFont[AttrBold] = PrnFont[AttrDefault];
    PrnFont[AttrBold | AttrUnder] = PrnFont[AttrUnder];
  }
  /* Special font */
  Prnlf.lfWeight = FW_NORMAL;
  Prnlf.lfUnderline = 0;
  Prnlf.lfWidth = FW; /* adjust width */
  Prnlf.lfHeight = FH;
  Prnlf.lfCharSet = 1;

  strcpy(Prnlf.lfFaceName,"Tera Special");
  PrnFont[AttrSpecial] = CreateFontIndirect(&Prnlf);
  PrnFont[AttrSpecial | AttrBold] = PrnFont[AttrSpecial];
  PrnFont[AttrSpecial | AttrUnder] = PrnFont[AttrSpecial];
  PrnFont[AttrSpecial | AttrBold | AttrUnder] = PrnFont[AttrSpecial];

  Black = RGB(0,0,0);
  White = RGB(255,255,255);
  SetTextColor(vpv->PrintDC, Black);
  SetBkColor(vpv->PrintDC,White);
  for (i = 0 ; i <= 255 ; i++)
    Dx[i] = FW;

  TempAttr = AttrDefault;
  TempAttr2 = AttrDefault2;
  Y = Mergin.top;

  TmpPtr = GetLinePtr(vpv,vpv->PrintStart.y);
  for (j = vpv->PrintStart.y ; j <= vpv->PrintEnd.y ; j++)
  {
    if (j==vpv->PrintStart.y)
      IStart = vpv->PrintStart.x;
    else IStart = 0;
    if (j == vpv->PrintEnd.y)
      IEnd = vpv->PrintEnd.x - 1;
    else
      IEnd = vpv->NumOfColumns - 1;

    if (Y+FH > Mergin.bottom)
    {
      /* next page */
      EndPage(vpv->PrintDC);
      StartPage(vpv->PrintDC);
      Y = Mergin.top;
    }

    X = Mergin.left;
    RText.top = Y;
    RText.bottom = Y + FH;
    i = IStart;
    do {
      CurAttr = vpv->AttrBuff[TmpPtr+i] & ~ AttrKanji;
      CurAttr2 = vpv->AttrBuff2[TmpPtr+i];

      count = 0;
      while ((i+count <= IEnd) &&
             (CurAttr == (vpv->AttrBuff[TmpPtr+i+count] & ~ AttrKanji)) &&
             (CurAttr2 == vpv->AttrBuff2[TmpPtr+i+count]) ||
             ((vpv->AttrBuff[TmpPtr+i+count-1] & AttrKanji) != 0))
        count++;

      if ((CurAttr != TempAttr) ||
          (CurAttr2 != TempAttr2))
      {
        SelectObject(vpv->PrintDC, PrnFont[CurAttr & AttrFontMask]);

        if ((CurAttr & AttrReverse) != 0)
        {
          SetTextColor(vpv->PrintDC,White);
          SetBkColor(  vpv->PrintDC,Black);
        }
        else {
          SetTextColor(vpv->PrintDC,Black);
          SetBkColor(  vpv->PrintDC,White);
        }

        TempAttr = CurAttr;
        TempAttr2 = CurAttr2;
      }

      RText.left = X;
      RText.right = X + count*FW;
      ExtTextOut(vpv->PrintDC,X,Y,6,&RText,&(vpv->CodeBuff[TmpPtr+i]),
                 count,&Dx[0]);

      X = X + count*FW;
      i = i+count;
    } while (i <= IEnd);

    Y = Y + FH;
    TmpPtr = NextLinePtr(vpv,TmpPtr);
  }

  EndPage(vpv->PrintDC);

  for (i = 0 ; i <= AttrFontMask ; i++)
  {
    for (j = i+1 ; j <= AttrFontMask ; j++)
      if (PrnFont[j]==PrnFont[i])
        PrnFont[j] = NULL;
    if (PrnFont[i] != NULL) DeleteObject(PrnFont[i]);
  }

}

